@@ -58,6 +58,9 @@ import pandas as pd
 from django.contrib.auth import get_permission_codename
 import re
 import html
+from ..django import get_list
+from ..django import get_key_from_request_data_or_self_obj
+from ..django.utils import get_field_type_in_db, get_field_type_in_py
 
 
 IAdmin = admin.ModelAdmin
@@ -219,8 +222,12 @@ class BulkDeleteMixin:
 
 
 class ExportExcelMixin:
-    export_asc = False      # 按id升序导出
-    add_index = False
+    export_ordering = None      # 导出时的排序
+    export_fields = None        # 需要导出的字段
+    extra_fields_dc = None      # 最后的verbose_name转换字典, 如: { 'ming_cheng': '哈哈哈名称' }
+
+    export_asc = False      # 按id升序导出 --- 不怎么需要这个字段了
+    add_index = False       # 导出时增加索引列
 
     def export_as_excel(self, request, queryset=None, model=None, extra_fields_dc=None, add_index_column=0, ordering=None):
         from django.db import models as m
@@ -236,121 +243,77 @@ class ExportExcelMixin:
             meta = model._meta
             queryset = model.objects.all()
 
-        # md = get_base_model(queryset)
-        field_names = [field.name for field in meta.fields]
-        verbose_names = [field.verbose_name for field in meta.fields]
+        request_data = request.POST
+        _selected_action = get_list(request_data, '_selected_action')
+        if _selected_action:
+            queryset = queryset.filter(pk__in=_selected_action)
+
+        _ordering = get_key_from_request_data_or_self_obj(request_data, self, key='export_ordering', get_type='list')
+        ordering = ordering if ordering else _ordering      # 函数优先, 其次查询前端请求和类属性
+        if ordering:
+            queryset = queryset.order_by(*ordering)
+
+        export_fields = get_key_from_request_data_or_self_obj(request_data, self, key='export_fields', get_type='list')
+
+        if export_fields:
+            field_names = [field.name for field in meta.fields if field.name in export_fields]
+            verbose_names = [field.verbose_name for field in meta.fields if field.name in export_fields]
+        else:
+            field_names = [field.name for field in meta.fields]
+            verbose_names = [field.verbose_name for field in meta.fields]
+
         conv_name_to_verbose_dc = dict(zip(field_names, verbose_names))
+
+        extra_fields_dc = self.extra_fields_dc if self.extra_fields_dc else extra_fields_dc
         if extra_fields_dc:
             conv_name_to_verbose_dc.update(extra_fields_dc)
 
-        if 1:
-            response = HttpResponse(content_type='application/msexcel')
-            filename = urlquote(f"{meta.verbose_name}.xls")
-            response['Content-Disposition'] = f'attachment; filename={filename}'
-            wb = Workbook()
-            ws = wb.active
+        response = HttpResponse(content_type='application/msexcel')
+        filename = urlquote(f"{meta.verbose_name}.xls")
+        response['Content-Disposition'] = f'attachment; filename={filename}'
+        wb = Workbook()
+        ws = wb.active
 
-            if self.export_asc:
-                if queryset.count() and hasattr(queryset[0], 'id'):
-                    queryset = queryset.order_by('id')
-            ws.append(verbose_names)
+        if self.export_asc:     # 不怎么需要这个字段了
+            queryset = queryset.order_by('pk')
 
-            total = queryset.count()
-            tq = tqdm(total=total)
-            for obj in queryset:
-                tq.update(1)
+        ws.append(list(conv_name_to_verbose_dc.values()))
 
-                # data = [f'{getattr(obj, field)}' for field in field_names]
-                data = []
-                for field_index in range(len(meta.fields)):
-                    _meta_field = meta.fields[field_index]
-                    field_name = _meta_field.name
-                    from ..django.utils import get_field_type_in_db, get_field_type_in_py
+        total = queryset.count()
+        tq = tqdm(total=total)
+        for obj in queryset:
+            tq.update(1)
 
-                    if isinstance(_meta_field, m.ForeignKey):
-                        # print('外键!', field_name, get_field_type_in_db(obj, field_name))
-                        to_field = _meta_field.to_fields[0]
-                        to_field = to_field if to_field else 'pk'
-                        field_value = getattr(obj, field_name)
-                        field_value = getattr(field_value, to_field)
-                    else:
-                        field_value = getattr(obj, field_name)
+            data = []
+            for field_index in range(len(meta.fields)):
+                _meta_field = meta.fields[field_index]
+                field_name = _meta_field.name
+                if export_fields and field_name not in export_fields:
+                    continue
 
-                    if get_field_type_in_py(obj, field_name) in ['int', 'float', 'bool']:
-                        data.append(field_value)
-                    # if get_field_type_in_py(obj, field_name) == 'bool':
-                    #     data.append(field_value)
-                    else:
-                        data.append(f'{field_value}' if field_value else field_value)
+                if isinstance(_meta_field, m.ForeignKey):
+                    # print('--- 外键: ', field_name, get_field_type_in_db(obj, field_name))
+                    to_field = _meta_field.to_fields[0]
+                    to_field = to_field if to_field else 'pk'
+                    field_value = getattr(obj, field_name)
+                    field_value = getattr(field_value, to_field)
+                else:
+                    field_value = getattr(obj, field_name)
 
-                try:
-                    ws.append(data)
-                except Exception as e:
-                    print(data)
-                    raise e
-            wb.save(response)
-            return response
+                if get_field_type_in_py(obj, field_name) in ['int', 'float', 'bool']:
+                    data.append(field_value)
+                # if get_field_type_in_py(obj, field_name) == 'bool':
+                #     data.append(field_value)
+                else:
+                    data.append(f'{field_value}' if field_value else field_value)
 
-        # if model is None:
-        else:
-            """
-            导出全部的时候, 大文件用流媒体传输, 并分批次读入, 防止超过云服务器带宽.
-            未完待续...
-            """
-            from django.http import StreamingHttpResponse
-            from django_pandas import io
-            from bdtime import tt
-            from django.db import models as m
-            # print('~~~~~~~~~~~~ queryset.count():', queryset.count())
-            qs: m.QuerySet = queryset
-
-            tt.__init__()
-
-            # print('开始读入sql...')
-            qs: m.QuerySet
-            # import pandas as pd
-            # from django.db import connection
-            # df = pd.concat([df for df in pd.read_sql(f'SELECT * FROM {meta.db_table}', connection, chunksize=100)], axis=0)
-            df = io.read_frame(qs=qs)
-            df.columns = [conv_name_to_verbose_dc.get(col) if conv_name_to_verbose_dc.get(col) else col for col in df.columns]
-            # print('df读入成功: ', tt.now(2))
-
-            if add_index_column:
-                # 增加一个序号列, start为 add_index_column - 1
-                index = pd.Series(df.index) + int(add_index_column) - 1    # add_index_column为2, 则从1开始
-                df.insert(loc=0, column='序号', value=index, allow_duplicates=False)
-
-            if ordering:
-                df = df[ordering]
-
-            # print('开始保存为excel...', tt.now(2))
-            time_str = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
-            tmp_file_name = f"{meta.verbose_name}_{time_str}.xlsx"
-            fpath = os.path.join(TEMPDIR, tmp_file_name)
-            # df.to_excel(fpath, index=False, encoding='unicode_escape')       # unicode_escape
-            df.to_excel(fpath, index=self.add_index)       # unicode_escape
-            # print('保存为Excel成功!', tt.now(2))
-            del df
-
-            # big file download
-            def file_iterator(file_name, open_model='rb', chunk_size=512):
-                with open(file_name, open_model) as f:      # , encoding=encoding
-                    while True:
-                        c = f.read(chunk_size)
-                        if c:
-                            yield c
-                        else:
-                            break
-
-            # print('write response', tt.now(2))
-            response = StreamingHttpResponse(file_iterator(fpath))
-            # response['Content-Type'] = 'application/octet-stream'
-            filename = urlquote(f"{meta.verbose_name}.xlsx")
-            response['Content-Type'] = 'application/msexcel'
-            response['Content-Disposition'] = 'attachment;filename="{0}"'.format(filename)
-
-            print('return response', tt.now(2))
-            return response
+            try:
+                ws.append(data)
+            except Exception as e:
+                print(data)
+                raise e
+        wb.save(response)
+        return response
 
     export_as_excel.short_description = "导出所选数据"
     # export_as_excel.acts_on_all = True
@@ -397,6 +360,9 @@ class ImportAdmin(IDAdmin):
     check_import_and_export_perm = True     # 是否检查导入导出按钮的权限
     use_original_id = True     # 是否保留原表格的id
 
+    use_bulk_create = False     # 是否使用`bulk_create`加快新建数据的速度, 但同时也会不调用`model.save()`方法
+    export_all_fields_forever = False       # 是否一直导出所有字段(右上角的`导出全部`和`action`中的导出数据做区分)
+
     def import_csv(self, request):
         t_import = Time()
         index = 0
@@ -455,8 +421,17 @@ class ImportAdmin(IDAdmin):
                 meta = self.model._meta
                 field_names = [field.name for field in meta.fields]
                 verbose_names = [field.verbose_name for field in meta.fields]
-                field_dc = dict(zip(verbose_names, field_names))
-                valid_columns = [column_i in field_names or column_i in verbose_names for column_i in df.columns]
+
+                # --- 兼容导出时自定义的列名
+                if hasattr(self, 'extra_fields_dc') and getattr(self, 'extra_fields_dc'):
+                    _field_dc = dict(zip(field_names, verbose_names))
+                    _field_dc.update(self.extra_fields_dc)
+                    field_dc = {v: k for k,v in _field_dc.items()}
+                else:
+                    field_dc = dict(zip(verbose_names, field_names))
+
+                new_verbose_names = list(field_dc.keys())
+                valid_columns = [column_i in field_names or column_i in new_verbose_names for column_i in df.columns]
                 df = df.loc[:, valid_columns]
                 titles = df.columns.tolist()
                 title_ls = [field_dc.get(title_i) if field_dc.get(title_i) else title_i for title_i in titles]
@@ -514,17 +489,27 @@ class ImportAdmin(IDAdmin):
 
                     dc = dict(zip(title_ls, content_ls))
 
-                    if not (self.use_original_id and set(df1.columns).intersection(set(['id', 'ID']))):
+                    if not (self.use_original_id and set(df1.columns).intersection(set(['id', 'ID', 'pk', 'PK']))):
                         # 如果[不使用原始id 并且 表格中有`id`列],  则启用自定义id
-                        dc.update({'id': curr_id})
+                        dc.update({'pk': curr_id})
                         curr_id += 1
 
                     md = self.model(**dc)
                     md_ls.append(md)
                     my_tqdm.update(1)
-                    # self.model.objects.create(**dc)
 
-                self.model.objects.bulk_create(md_ls,batch_size=100)
+                    # if self.use_bulk_create is False:
+                    #     self.model.objects.create(**dc)
+
+                if self.use_bulk_create:
+                    self.model.objects.bulk_create(md_ls, batch_size=100)
+                else:
+                    save_tqdm = tqdm(total=df_rows)
+                    save_tqdm.desc = '写入数据库ing...'
+                    for md_i in md_ls:
+                        save_tqdm.update(1)
+                        md_i.save()
+
                 reset_db_sequence(self.model)
                 self.message_user(request, f"{f_format}文件导入成功! 一共导入{df_rows}条数据, 耗时: {t_import.now(1)}秒.")
                 self.remove_temp_file(tempdir)
@@ -571,11 +556,18 @@ class ImportAdmin(IDAdmin):
 
     def export_all_csv(self, request):
         # self.message_user(request, "成功导出全部数据为csv文件")
-        return ExportCsvMixin().export_as_csv(request, queryset=self.model.objects.all(), model=self.model)
+        if hasattr(self, 'export_as_csv') and self.export_all_fields_forever is False:
+            ret = self.export_as_csv(request, queryset=self.model.objects.all(), model=self.model)
+        else:
+            ret = ExportCsvMixin().export_as_csv(request, queryset=self.model.objects.all(), model=self.model)
+        return ret
 
     def export_all_excel(self, request):
         # self.message_user(request, "成功导出全部数据为csv文件")
-        ret = ExportExcelMixin().export_as_excel(request, queryset=self.model.objects.all(), model=self.model)
+        if hasattr(self, 'export_as_excel') and self.export_all_fields_forever is False:
+            ret = self.export_as_excel(request, queryset=self.model.objects.all(), model=self.model)
+        else:
+            ret = ExportCsvMixin().export_as_excel(request, queryset=self.model.objects.all(), model=self.model)
         self.remove_temp_file(TEMPDIR)
         return ret
 
